(ns wanijo.attribute.domain
  (:require [clojure.spec.alpha :as spec]
            [wanijo.framework.neo4j :as neo4j]))

(spec/def ::name
  (spec/and string? not-empty))

(spec/def ::type
  (spec/and
   string?
   #(some (partial = %) ["date" "file", "string" "text"])))

(spec/def ::required
  #(some (partial = %) ["on" nil 0 1 "0" "1"]))

(spec/def ::created-at
  (spec/and string? ::neo4j/date-str))

(neo4j/defquery
  findy-by-schema
  "MATCH (a:attribute)-->(s:schema)
   WHERE s.uuid = {uuid}
   RETURN a
   ORDER BY a.name")

(defn find-by-schema! [schema-uuid]
  (->>
   (neo4j/exec-query!
    findy-by-schema
    {:uuid schema-uuid})
   (map :a)))

(neo4j/defquery create-new
  "MATCH (u:user)--(s:schema)
   WHERE u.uuid = {user_uuid}
     AND s.uuid = {schema_uuid}
   CREATE (a:attribute)
   CREATE (a)-[:created_by]->(u)
   CREATE (a)-[:of]->(s)
      SET a.type = {type}
      SET a.name = {name}
      SET a.uuid = {attribute_uuid}
      SET a.required = {required}
      SET a.created_at = {created_at}")

(defn create-new! [attr schema-uuid user-uuid]
  (neo4j/exec-query!
   create-new
   (-> attr
       (assoc
        :user_uuid user-uuid
        :schema_uuid schema-uuid
        :attribute_uuid (neo4j/uuid)
        :created_at (neo4j/now-str))
       (update :required #(if (some? %) 1 0)))))

(neo4j/defquery edit
  "MATCH (a:attribute)
   WHERE a.uuid = {uuid}
     SET a.type = {type}
     SET a.name = {name}
     SET a.required = {required}")

(defn edit! [attr]
  (neo4j/exec-query!
   edit
   (update attr
           :required #(if (some? %) 1 0))))

(neo4j/defquery delete-by-uuid
  "MATCH (a:attribute)-[c]-()
   WHERE a.uuid = {uuid}
   DELETE c, a")

(defn delete-by-uuid! [uuid]
  (neo4j/exec-query!
   delete-by-uuid
   {:uuid uuid}))
