(ns wanijo.framework.neo4j
  (:require [neo4j-clj.core :as db]
            [wanijo.framework.devmode :as devmode]
            [clj-time.format :as time-format]
            [clj-time.local :as time-local]
            [clojure.spec.alpha :as spec])
  (:import (java.util UUID)))

(spec/def ::date-str
  #(re-matches #"\d{4}\d{2}\d{2}T\d{2}\d{2}\d{2}\.\d{3}Z" %))

(def conn
  (delay (db/connect
          "bolt://localhost:7687"
          "neo4j"
          "b")))

(defmacro defquery [& args] `(db/defquery ~@args))

(defn uuid []
  (str (UUID/randomUUID)))

(defn butiful-query [qry]
  (->> qry
       str
       clojure.string/trim-newline
       clojure.string/split-lines
       (map clojure.string/trim)
       (filter #(pos? (count %)))
       (clojure.string/join \newline)))

(defn exec-query! [qry params]
  (with-open [session (db/get-session @conn)]
    (devmode/send-to-bar
     (str (butiful-query qry)
          "<br>---Params---<br>"
          params))
    (qry session params)))

(defn now-str []
  (time-format/unparse
   (time-format/formatters :basic-date-time)
   (time-local/local-now)))
